#!/bin/csh
#
# zeus32.s
#
# This script does everything necessary to create the Zeus-3D executable 
# xzeus32 on various systems.
#
# Laboratory for Computational Astrophysics
# National Center for Supercomputing Applications
# University of Illinois at Urbana-Champaign
#
# Written by Robert Fiedler 11/03/93
#
# This script works automatically for the following machines (hosts):
#
#   c3   Convex 3880
#   uy   Cray Y-MP
#   u2   Cray 2 (NCSA's Cray 2 has been shut down permanently)
#
# When used with other hosts, the user is asked to identify
# the operating system (CONVEXOS || UNICOS, UY || UNICOS, U2).
#
# To create the executable xzeus32 on c3 or uy, the following files are needed
# and will be retrieved if necessary by this script via AFS or ftp:
#
# zeus32.s          this script
# zeus32.mac        macro definitions (one version is included below)
# zeus32            Zeus-3D source code
# xedit21           EDITOR executable to edit files
# namelist.lib      library to simulate FORTRAN namelist
# cfiles.lib        C program that checks for tty input
# ncar.lib(o)       library of customized NCAR routines
#
# For CONVEXOS, a modified version of the NCAR graphics loader
# script "ncargf77" is supplied to prevent a "word too long" error.
#
# The above files (or scripts to generate them) are stored on AFS and will be 
# copied to the current working directory by this script if they don't exist 
# there already.
#
# For uy and other machines without working AFS client software, files
# must be retrieved using ftp to "landrew".  In this case, you may need to get
# a temporary password to the account 'zeus3d' on landrew from the LCA.
#
#  [AFS Tip: It is convenient to alias the command used to connect to
#   AFS so that an environment variable CONN gets set.  One might put:
#
#   alias rklog   "\rklog \!* ; setenv CONN"
#
#   ..in the .cshrc file, for example, but this is not necessary.]
#
# Note that the libraries namelist.lib, cfiles.lib, and ncar.lib(o) 
# are created if needed from their respective source codes by invoking the
# scripts namelist.s, cfiles.s, and ncarlib(o).s.  The source codes 
# for these libraries should be compiled with the same 
# optimization options as those used for the Zeus-3D source code.  
#
# For "unknown" machines (other than c3, uy, u2), this script first 
# gets and runs the script 'edit21.s' to generate the xedit21 executable.
# 
#.....................................................................
#
# Identify the host, operating system, and means of accessing files on AFS.
#
set HOST = `hostname | sed -e 's/\..*//'`
switch ($HOST)
case c3:
  set OSYS = "CONVEXOS"
  set OP_SYS = "CONVEXOS"
  set ACCESS = "AFS"
  breaksw
case uy:
  set OSYS = "UNICOS"
  set OP_SYS = "UNICOS, UY"
  set ACCESS = "ftp"
  breaksw
case u2:
  set OSYS = "UNICOS"
  set OP_SYS = "UNICOS, U2"
  set ACCESS = "ftp"
  breaksw
default:
#
# Ask the user about the system, access to AFS, and signon for AFS.
#
  echo ""
  echo -n "Please identify your system (CONVEXOS || UNICOS, UY || UNICOS, U2): "
  set OP_SYS = ( $< )
  set OSYS = `echo $OP_SYS[1] | sed -e 's/,//'`
  echo -n "...and your method of copying files from mass storage (AFS || ftp): "
  set ACCESS = $<
  if ($ACCESS == "ftp") then
    echo -n "Please type your signon for NCSA's AFS server landrew ($LOGNAME): "
    set User = $<
    if ($User == "") set User = $LOGNAME
  endif
  set HOST = "unknown"
  echo ""
endsw
#
# Set up directory names for the mass storage system.
#
  set COPY = "get"
  if ($ACCESS == "AFS") set COPY = "cp"
  set ZEUS3D = "/afs/ncsa/projects/lca/codes/zeus3d"
  set EDITOR = "$ZEUS3D/editor"
  set NCAR   = "$ZEUS3D/libr/ncar"
endif
#
# To help keep compiler options the same between scripts, we define a 
# set of environment variables for FORTRAN compiler, C compiler, 
# and loader options.  If these compiler options are changed, the 
# libraries namelist.lib, cfiles.lib, and ncar.lib(o) should be 
# recreated by rerunning the appropriate scripts. 
#
switch ($OSYS)
case CONVEXOS:
  if (! $?F_COMP_OPTS) setenv F_COMP_OPTS  '-fi -O2 -nw -or none -db'
  if (! $?C_COMP_OPTS) setenv C_COMP_OPTS  '-fi -O2 -db'
  if (! $?LDR_OPTS)    setenv LDR_OPTS     ''
  set LDR = './ncargf77'
  set NCAR_LIB = "libo"
  set LIBS = "namelist.lib cfiles.lib ncar.${NCAR_LIB} -ldf -lveclib"
  breaksw
case UNICOS:
  if (! $?F_COMP_OPTS) setenv F_COMP_OPTS  '-M0726 -ez'
  if (! $?C_COMP_OPTS) setenv C_COMP_OPTS  ''
  if (! $?LDR_OPTS)    setenv LDR_OPTS     ''
  set LDR = 'ncargf77'
  set NCAR_LIB = "lib"
  set LIBS = "-lsci -ldf namelist.lib cfiles.lib ncar.${NCAR_LIB}"
  breaksw
default:
#
# (Same as CONVEXOS)
#
  if (! $?F_COMP_OPTS) setenv F_COMP_OPTS  '-fi -O2 -nw -or none -db'
  if (! $?C_COMP_OPTS) setenv C_COMP_OPTS  '-fi -O2 -db'
  if (! $?LDR_OPTS)    setenv LDR_OPTS     ''
  set LDR = './ncargf77'
  set NCAR_LIB = "libo"
  set LIBS = "namelist.lib cfiles.lib ncar.${NCAR_LIB} -ldf -lveclib"
  breaksw
endsw
#
echo ""
echo "FORTRAN compiler options are: $F_COMP_OPTS"
echo "C compiler options are:       $C_COMP_OPTS"
echo "Loader is:                    $LDR"
echo "Loader options are:           $LDR_OPTS"
echo "LIBS are: $LIBS"
echo ""
#
# The system is now identified and machine-specific variables are now set.
#
#.....................................................................
#
# See that all files needed to produce xzeus32 are on local disk.
#
if (! (-e zeus32 && -e xedit21 && \
    -e namelist.lib && -e cfiles.lib && -e ncar.$NCAR_LIB && \
    ($OSYS != "CONVEXOS" || ($OSYS == "CONVEXOS" && -e ncargf77)))) then
#
# Write GETS script to get the absent files from mass storage.
#
  echo "#\!/bin/csh" > GETS
  if ($ACCESS == "ftp") then
    echo "ftp -n landrew.ncsa.uiuc.edu << EOF" >> GETS
    echo "user $User" >> GETS
    set D = ""
  else
    set D = ${cwd}/
  endif
  if (! -e zeus32) then
    echo "cd $ZEUS3D/source/version3.2" >> GETS
    echo "$COPY zeus32 ${D}zeus32" >> GETS
  if (! -e xedit21) then
    if ($HOST != "unknown") then
      echo "cd $EDITOR/xeq/version2.1/$HOST" >> GETS
      echo "$COPY xedit21 ${D}xedit21" >> GETS
    else
      echo "cd $EDITOR/source/version2.1" >> GETS
      echo "$COPY edit21.s ${D}edit21.s" >> GETS
      echo "$COPY edit21 ${D}edit21" >> GETS
      echo "$COPY edit21.f ${D}edit21.f" >> GETS
      echo "cd $EDITOR/source/namelist" >> GETS
      echo "$COPY namelist ${D}namelist" >> GETS
      echo "$COPY namelist.f ${D}namelist.f" >> GETS
      echo "$COPY namelist.s ${D}namelist.s" >> GETS
    endif
  endif
  if (! -e namelist.lib) then
    if ($HOST != "unknown") then
      echo "cd $EDITOR/xeq/namelist/$HOST" >> GETS
      echo "$COPY namelist.lib ${D}namelist.lib" >> GETS
    else
      if (-e xedit21) then
        echo "cd $EDITOR/source/namelist" >> GETS
        echo "$COPY namelist ${D}namelist" >> GETS
        echo "$COPY namelist.s ${D}namelist.s" >> GETS
      endif
    endif
  endif
  endif
  if (! -e cfiles.lib) then
    if ($HOST != "unknown") then
      echo "cd $ZEUS3D/cfiles/$HOST" >> GETS
      echo "$COPY cfiles.lib ${D}cfiles.lib" >> GETS
    else
      if ($OSYS == "CONVEXOS") then
        echo "cd $ZEUS3D/cfiles/c3" >> GETS
        echo "$COPY checkin.c ${D}checkin.c" >> GETS
        echo "$COPY cfiles.s ${D}cfiles.s" >> GETS
      else
#
# *** Need a script to make cfiles.lib for UNICOS.
#
        echo ""
        echo "Sorry, this script cannot make cfiles.lib for this machine."
        echo ""
        exit
      endif
    endif
  endif
  if (! -e ncar.$NCAR_LIB) then
    if ($HOST != "unknown") then
      echo "cd $NCAR/$HOST" >> GETS
      echo "$COPY ncar.$NCAR_LIB ${D}ncar.$NCAR_LIB" >> GETS
    else
      echo "cd $NCAR/source" >> GETS
      echo "$COPY ncar${NCAR_LIB} ${D}ncar${NCAR_LIB}" >> GETS
      echo "$COPY ncar${NCAR_LIB}.s ${D}ncar${NCAR_LIB}.s" >> GETS
    endif
  endif
  if (! -e xedit21 || ! -e namelist.lib || ! -e cfiles.lib || \
      ! -e ncar.$NCAR_LIB) then
    echo "cd $ZEUS3D/util" >> GETS
    echo "$COPY bld_lib ${D}bld_lib" >> GETS
  endif
  if ($OSYS == "CONVEXOS") then
    if (! -e ncargf77) then
      echo "cd $NCAR/c3" >> GETS
      echo "$COPY ncargf77 ${D}ncargf77" >> GETS
    endif
  endif
  echo "EOF" >> GETS
  echo "exit" >> GETS
#
# Connect to mass storage.
#
  echo ""
  echo "Getting files from mass storage ($ACCESS - landrew)."
  echo ""
  if (! $?CONN && $COPY == "cp") then
    echo ""
    echo -n "Are we connected to AFS (y/n)? "
    setenv CONN $<
    if ($CONN == "n") then
      echo "Enter your AFS password at the prompt."
      echo ""
      rklog
      setenv CONN "y"
    endif
  endif
  chmod 700 GETS
  GETS
  echo ""
  \rm -f GETS
#
# Run the scripts to create xedit21 and the libraries.
#
  if (! -e xedit21) then
    echo ""
    echo "Creating xedit21"
    echo ""
    chmod 700 edit21.s
    edit21.s
  endif
  if (! -e namelist.lib) then
    echo ""
    echo "Creating namelist.lib"
    echo ""
    chmod 700 namelist.s
    namelist.s
  endif
  if (! -e cfiles.lib) then
    echo ""
    echo "Creating cfiles.lib"
    echo ""
    chmod 700 cfiles.s
    cfiles.s
  endif
  if (! -e ncar.$NCAR_LIB) then
    echo ""
    echo "Creating ncar.$NCAR_LIB"
    echo ""
    ncar${NCAR_LIB}.s
    echo ""
  endif
else
  echo "No files needed from mass storage."
  echo ""
endif
#
# For Convex, update table of contents for the libraries.
#
if ($OSYS == "CONVEXOS") then
  ranlib namelist.lib
  ranlib cfiles.lib
  ranlib ncar.$NCAR_LIB
endif
#
# All the necessary files are now on local disk.
#
#.......................................................................
#
# Create the EDITOR input decks and run EDITOR to produce the Zeus-3D
# FORTRAN source code and the makefile.
#
echo ""
echo "Creating the executable xzeus32."
echo ""
#
# If needed, create a directory to hold the source code modules.
#
if (! -e zeus3.2) mkdir zeus3.2
#
# Create a new change deck chgzeus if one does not exist.  The one below sets
# up a 64-cubed grid (excluding ghost zones).
#
if (! -e chgzeus) then
  echo "Writing new change deck chgzeus:"
  echo ""
  cat << EOF > chgzeus
*read zeus32.mac
*d par.14,15
       parameter     ( in   =  69, jn   =  69, kn   =  69, ijkn =  69 )
       parameter     ( nxpx =   1, nypx =   1, nxrd =   1, nyrd =   1 )
EOF
else
  echo "Using existing change deck chgzeus:"
  echo ""
endif
#
cat chgzeus
echo ""
#
# If needed, create a new input deck for EDITOR to write the makefile 
# makezeus and preprocess the Zeus-3D source code zeus32.
#
if (! -e inedit_zeus32) then
  echo "Writing new EDITOR input deck inedit_zeus32:"
  echo ""
  cat << EOF > inedit_zeus32
 \$editpar   inname='zeus32'
          , ibanner=0, idump=1, job=3, safety=0.20
          , ipre=1, inmlst=1, iupdate=1, iutask=0
          , chgdk='chgzeus'
          , branch='zeus3.2'
          , makename='makezeus', xeq='xzeus32', loader='$LDR'
          , coptions='$F_COMP_OPTS'
          , loptions='$LDR_OPTS'
          , libs='$LIBS' \$
EOF
else
  echo "Using existing EDITOR input deck inedit_zeus32:"
  echo ""
endif
cp inedit_zeus32 inedit
#
cat inedit
#
if (! -e zeus32.mac) then
  echo ""
  echo "Writing new macro definition file zeus32.mac:"
  echo ""
#
# The zeus32.mac file below is for "blast", the expansion of a hot sphere
# of plasma in a rarefied medium with magnetic fields.
#
  cat << EOF > zeus32.mac
**==*====1====*====2====*====3====*==*====3====*====2====*====1====*====
**                                                                    **
******************  CONDITIONAL COMPILATION SWITCHES  ******************
**                                                                    **
**  1) symmetry axes:  ISYM, JSYM, KSYM
**
**define   KSYM, JSYM
**
**  2) geometry:  XYZ, or ZRP, or RTP
**
*define   XYZ
**
**  3) physics:  MHD, ISO, GRAV
**
*define   MHD
**
**  4) algorithms:  MOC
**
*define   MOC
**
**  5) data output modes:  PLT1D, PLT2D, PIX, VOX, HDF, DISP, RADIO,
**                         TIMESL
**
*define   DISP, TIMESL
**
**  6) operating system:  UNICOS, UY, U2, CONVEXOS
**
*define   $OP_SYS
**
*************************  MODULE NAME ALIASES  ************************
**                                                                    **
**  The modules "BNDYUPDATE", "SPECIAL", "SPECIALSRC", "SPECIALTRN",
**  "USERDUMP", and "FINISH" are slots available to the user to help
**  adapt the code to the problem being solved.
**
*alias    START            mstart
*alias    BNDYUPDATE       empty
*alias    EXTENDGRID       empty
*alias    GRAVITY          empty
*alias    SPECIAL          empty
*alias    SOURCE           srcstep
*alias    SPECIALSRC       empty
*alias    TRANSPORT        transprt
*alias    SPECIALTRN       empty
*alias    NEWTIMESTEP      nudt
*alias    NEWGRID          empty
*alias    DATAOUTPUT       dataio
*alias    FINISH           empty
**
*alias    PROBLEM          blast
*alias    ATMOSPHERE       empty
*alias    PROBLEMRESTART   empty
*alias    ARTIFICIALVISC   viscous
*alias    USERDUMP         empty
**                                                                    **
************************  ERROR CRITERIA ALIASES  **********************
**                                                                    **
*alias    GRAVITYERROR     1.0e-6
*alias    GRIDERROR        1.0e-6
*alias    PDVCOOLERROR     1.0e-6
*alias    NEWVGERROR       1.0e-10
**                                                                    **
***********************  ITERATION LIMITS ALIASES  *********************
**                                                                    **
*alias    GRAVITYITER      4
*alias    GRIDITER         20
*alias    PDVCOOLITER      20
*alias    NEWVGITER        20
EOF
else
  echo ""
  echo "Using existing zeus32.mac:"
  echo ""
endif
cat zeus32.mac
#
echo ""
echo "Invoking EDITOR."
echo ""
chmod 700 xedit21 
xedit21
echo ""
\rm -f inedit
#
echo "Invoking make."
echo ""
make -f makezeus
#
# The Zeus-3D executable xzeus32 should exist now.
#
# If necessary, create the input deck inzeus.  The one below sets
# up a 3-D grid from -1 to +1 in all directions.  The problem is the
# expansion of a hot sphere of plasma in a mangetic field.
#
if (! -e inzeus) then
  echo ""
  echo "Writing new zeus input file inzeus:"
  echo ""
  cat << EOF > inzeus
 \$iocon    iotty=6, iolog=2                                            \$
 \$rescon   dtdmp=0.1, id='za', resfile='zr00za'                        \$
 \$ggen1    nbl=032, x1min=-1.0, x1max= 0.0, igrid=-1, x1rat=1.02       \$
 \$ggen1    nbl=032, x1min= 0.0, x1max= 1.0, igrid= 1, x1rat=1.02
         , lgrid=.t.                                                   \$
 \$ggen2    nbl=032, x2min=-1.0, x2max= 0.0, igrid=-1, x2rat=1.02       \$
 \$ggen2    nbl=032, x2min= 0.0, x2max= 1.0, igrid= 1, x2rat=1.02
         , lgrid=.t.                                                   \$
 \$ggen3    nbl=032, x3min=-1.0, x3max= 0.0, igrid=-1, x3rat=1.02       \$
 \$ggen3    nbl=032, x3min= 0.0, x3max= 1.0, igrid= 1, x3rat=1.02
         , lgrid=.t.                                                   \$
 \$pcon     nlim= 999999, tlim=0.00001, ttotal=1000.0, tsave=10.0       \$
 \$hycon    qcon=2.0, qlin=0.0, courno=0.5, iord=2, istp=0, itote=0     \$
 \$iib      niib(1:69,1:69)=2                                           \$
 \$oib      noib(1:69,1:69)=2                                           \$
 \$ijb      nijb(1:69,1:69)=2                                           \$
 \$ojb      nojb(1:69,1:69)=2                                           \$
 \$ikb      nikb(1:69,1:69)=2                                           \$
 \$okb      nokb(1:69,1:69)=2                                           \$
 \$grvcon                                                               \$
 \$eqos     gamma=1.6666667                                             \$
 \$gcon                                                                 \$
 \$extcon                                                               \$
 \$plt1con                                                              \$
 \$plt2con                                                              \$
 \$pixcon                                                               \$
 \$voxcon                                                               \$
 \$usrcon                                                               \$
 \$hdfcon                                                               \$
 \$tslcon   dttsl=0.00001                                               \$
 \$discon   idisdir=1, ldis=35, dtdis=0.00001, disvar='p '
         ,           jdismn=35, jdismx=44, kdismn=35, kdismx=51        \$
 \$radcon                                                               \$
 \$pgen     r=0.20, x10=0.0, x20=0.0, x30=0.0, d0=1.0, drat=100.0
         , p0=0.6, prat=1.0e6, b10=140.0, b20=0.0, b30=0.0             \$
EOF
else
  echo ""
  echo "Using existing zeus input file inzeus:"
  echo ""
endif
cat inzeus
echo ""
#
# The problem "blast" can now be run by typing xzeus32.  Execution terminates
# upon reaching the time limit tlim after about 40 sec of Y-MP CPU time.
# Time slice and display dumps are produced for diagnostic purposes.
#
exit
