#!/bin/csh
#
# HYDRO_BM.s <N_Threads_max> <Algorithm> <Grid>
#
# Script to perform a ZEUS-3D blast benchmark on various platforms
# and tabulate the results.
#
# Algorithm is specified by one of the following integers:
#              0) HY
#              1) CT
#              2) MC (MoC)
#              3) HS (HSMoC)
#
# Grid is input in the form <NZ1>x<NZ2>x<NZ3>.  Valid choices are:
#                 32x32x32
#                 64x32x32
#                 64x64x32
#                 64x64x64
#                 128x64x64
#                 128x128x64
#                 128x128x128
#                 256x128x128
#                 256x256x128
#
# EXAMPLE
#
#   HYDRO_BM.s 16 0 32x32x32
#
#     Performs a series of runs on 1 to 16 threads,
#     hydro-only algorithm,
#     total work fixed at 32x32x32 zones.
#
#     In practice, HYDRO_BM.s is called by driver scripts
#     HYDRO_SCALED.s and HYDRO_FIXED.s.
#
# RAF 5/20/97
#.................................................................
# 
# Set problem parameters and MFLOP normalization from C90
#
if (! $?ID) setenv ID 001
if (! $?N_THREADS_MIN) setenv N_THREADS_MIN 1
if (! $?HOST_NAME) setenv HOST_NAME `hostname | sed -e 's/\..*//'`
echo ""
#
switch ($HOST_NAME)
  case modi1:
  case modi2:
  case modi3:
  case modi4:
    if ($#argv < 3 && ! $?N_THREADS_MAX) setenv N_THREADS_MAX 32
    set Fortran = f77
    breaksw
  case bragi:
  case freya:
  case thor:
  case odin:
  case sif:
    if ($#argv < 3 && ! $?N_THREADS_MAX) setenv N_THREADS_MAX 16
    set Fortran = f77
    breaksw
  case magna:
    if ($#argv < 3 && ! $?N_THREADS_MAX) setenv N_THREADS_MAX 32
    set Fortran = f77
    breaksw
  case loki:
    if ($#argv < 3 && ! $?N_THREADS_MAX) setenv N_THREADS_MAX 12
    set Fortran = f77
    breaksw
  case lena:
    if ($#argv < 3 && ! $?N_THREADS_MAX) setenv N_THREADS_MAX 16
    set Fortran = f77
    breaksw
  case ella:
  case tuvok:
    if ($#argv < 3 && ! $?N_THREADS_MAX) setenv N_THREADS_MAX 16
    set Fortran = f77
    breaksw
  case billie:
    if ($#argv < 3 && ! $?N_THREADS_MAX) setenv N_THREADS_MAX 32
    set Fortran = f77
    breaksw
  case dax:
    if ($#argv < 3 && ! $?N_THREADS_MAX) setenv N_THREADS_MAX 64
    set Fortran = f77
    breaksw
  case mario:
    if ($#argv < 3 && ! $?N_THREADS_MAX) setenv N_THREADS_MAX 1
    set Fortran = cf77
    breaksw
  case c3:
  case uxh:
    if ($#argv < 3 && ! $?N_THREADS_MAX) setenv N_THREADS_MAX 1
    set Fortran = fc
    breaksw
  case zeus:
  case athena:
  case space:
    if ($#argv < 3 && ! $?N_THREADS_MAX) setenv N_THREADS_MAX 1
    set Fortran = f77
    breaksw
  case nl-ibm1:
    if ($#argv < 3 && ! $?N_THREADS_MAX) setenv N_THREADS_MAX 1
    set Fortran = xlf
    breaksw
  default:
    if ($#argv < 3 && ! $?N_THREADS_MAX) then
      echo -n "Enter the maximum number of threads: "
      set N_THR_MAX = "$<"
      if ("$N_THR_MAX" != "") then
        setenv N_THREADS_MAX "$N_THR_MAX"
      else
        setenv N_THREADS_MAX 1
      endif
      echo -n "Enter the name of the Fortran compiler: "
      set Fortran = "$<"
    endif
    breaksw
endsw
if ($#argv >= 3) then
  setenv N_THREADS_MAX $1
  shift
endif
echo "N_THREADS_MAX = $N_THREADS_MAX"
#
# If the algorithm was not specified on the command line, assume HY.
#
if ($#argv < 2) then
  set ALG = 0
else
  set ALG = $1
endif
set DATA_DIR = "${HOME}/zeus3d_BM"
if (! -e $DATA_DIR) mkdir $DATA_DIR
if ($?SCALED) then
  set ZBM_N = "${DATA_DIR}/Z3S_B_"
else
  set ZBM_N = "${DATA_DIR}/Z3F_B_"
endif
switch ($ALG)
  case 0:
    echo "Hydro only"
    set ZBM_N = "${ZBM_N}HY"
    breaksw
  case 1:
    echo "CT algorithm"
    set ZBM_N = "${ZBM_N}CT"
    breaksw
  case 2:
    echo "MoC algorihm"
    set ZBM_N = "${ZBM_N}MC"
    breaksw
  case 3:
    echo "HSMoC algorithm"
    set ZBM_N = "${ZBM_N}HS"
    breaksw
endsw
#
# If the grid index was not specified on the command line, determine
# it from the environment variables NXZ*.
#
if ($#argv < 2) then
  if (! $?NZX1) setenv NZX1 128
  if (! $?NZX2) setenv NZX2 128
  if (! $?NZX3) setenv NZX3 128
  set GRID = ${NZX1}x${NZX2}x${NZX3}
else
  set GRID = $2
  setenv NZX1 `echo $GRID | sed -e 's/x.*//'`
  setenv NZX2 `echo $GRID | sed -e 's/[0-9]*x//' | sed -e 's/x.*//'`
  setenv NZX3 `echo $GRID | sed -e 's/.*x//'`
endif
echo "$GRID grid"
set UGRID = `echo $GRID | sed -e 's/x/_/g'`
if (! $?ZBM_Z) setenv ZBM_Z "${ZBM_N}_${UGRID}.$ID"
#
# Set C90 data for this experiment.  If this machine is mario,
# we will extract this data from the hpm output.
#
switch ($ALG)
  case 0:
#
# HY
#
    switch ($GRID)
      case 32x32x32:
        set C90_MF = 190.0
        set C90_ZC = 217382.0
        breaksw
      case 64x32x32:
        set C90_MF = 274.0
        set C90_ZC = 321223.0
        breaksw
      case 64x64x32:
        set C90_MF = 285.0
        set C90_ZC = 340912.0
        breaksw
      case 64x64x64:
        set C90_MF = 293.0
        set C90_ZC = 358036.0
        breaksw
      case 128x64x64:
        set C90_MF = 359.0
        set C90_ZC = 443360.0
        breaksw
      default:
        echo "No C90 data for this problem size; using 128x64x64 data."
        set C90_MF = 359.0
        set C90_ZC = 443360.0
    endsw
    breaksw
  case 1:
#
# CT
#
    switch ($GRID)
      case 1:
        set C90_MF = 113.0
        set C90_ZC = 105532.0
        breaksw
      case 2:
        set C90_MF = 219.0
        set C90_ZC = 217767.0
        breaksw
      case 3:
        set C90_MF = 299.0
        set C90_ZC = 291647.0
        breaksw
      case 4:
        set C90_MF = 241.0
        set C90_ZC = 239992.0
        breaksw
      default:
        echo "Choose 1 through 5 to specify a grid."
        stop
    endsw
    breaksw
  case 2:
#
# MoC
#
    switch ($GRID)
      case 1:
        set C90_MF = 137.0
        set C90_ZC = 59125.0
        breaksw
      case 2:
        set C90_MF = 252.0
        set C90_ZC = 109773.0
        breaksw
      case 3:
        set C90_MF = 288.0
        set C90_ZC = 122886.0
        breaksw
      case 4:
        set C90_MF = 268.0
        set C90_ZC = 116074.0
        breaksw
      case 5:
        set C90_MF = 356.21
        set C90_ZC = 140794.0
        breaksw
      case 6:
        set C90_MF = 400.0
        set C90_ZC = 150000.0
        breaksw
      default:
        echo "Choose 1 through 5 to specify a grid."
        stop
    endsw
    breaksw
  case 3:
#
# HSMoC
#
    switch ($GRID)
      case 1:
        set C90_MF = 135.0
        set C90_ZC = 51500.0
        breaksw
      case 2:
        set C90_MF = 241.0
        set C90_ZC = 92771.0
        breaksw
      case 3:
        set C90_MF = 278.0
        set C90_ZC = 104802.0
        breaksw
      case 4:
        set C90_MF = 259.0
        set C90_ZC = 99100.0
        breaksw
      default:
        echo "Choose 1 through 4 to specify a grid."
        exit 3
        breaksw
    endsw
    breaksw
  default:
    echo "Choose 0 through 3 to specify an algorithm."
    exit 1
    breaksw
endsw 
#
# Do a single-threaded run for normalization.  It should work.
# Skip this for SCALED exeriments and N_THREADS_MIN > 1.
#
set N_Threads = $N_THREADS_MIN
if (! $?SCALED || $N_THREADS_MAX == 1) then
#
# Run ZEUS-3D on N_Threads threads
#
setenv MP_SET_NUMTHREADS $N_Threads
setenv MP_NUMBER_OF_THREADS $N_Threads
echo ""
echo "N_Threads = $N_Threads"
if ("$HOST_NAME" == lena || "$HOST_NAME" == ella) then
  mpa xzeus34 > ZBM_out
else if ("$HOST_NAME" == dax || "$HOST_NAME" == billie) then
  mpa -DATA -STACK -private node_private -stacktype node_private xzeus34 > ZBM_out
else if ("$HOST_NAME" == mario) then
  (hpm xzeus34) >& ZBM_out
#
# Extract MFLOPS from ZBM_out and round to nearest whole number.
#
  set LINA = (`grep '^Floating ops\/sec' ZBM_out`)
  echo "$LINA"
  set C90_MF = `echo $LINA[4] | sed -e 's/M//'`
  set FRAC = `echo $C90_MF | sed -e 's/[0-9]*\.//'`
  set C90_MF = `echo $C90_MF | sed -e 's/\.[0-9]*//'`
  if ($FRAC >= 50) @ C90_MF = $C90_MF + 1
  set C90_MF = "${C90_MF}.0"
else
  xzeus34 > ZBM_out
endif
#
# Extract tused
#
set LINA = (`grep 'tused' ZBM_out | tail -1`)
echo "$LINA"
set TEST = `echo "$LINA" | grep '=-'`
if ("$TEST" != "") then
#
# Negative trem
#
  @ NLINA = $#LINA - 1
else
#
# Positive trem
#
  @ NLINA = $#LINA - 2
endif
set TUSED = `echo $LINA[$NLINA] | sed -e 's/tused=//'`
#
# Extract the zone-cycles/sec
#
set LINA = (`grep 'Average   zone-cycles' ZBM_out`)
echo "$LINA"
set ZCPS = $LINA[$#LINA]
if ("$HOST_NAME" == mario) set C90_ZC = $ZCPS
#
# Extract the wall clock time
#
set LINA = (`grep 'Wall clock time' ZBM_out`)
echo "$LINA"
set WALL = `echo $LINA[7] | sed -e 's/,//'`
#
# Write to the table
#
set TUSED1 = $TUSED
echo "" > $ZBM_Z
echo " Processors  Wall Clock  tused(s) Zone-Cycles/sec   MFLOPS   C90s   Speedup Speedup/Processor" >> $ZBM_Z
set ZCPS1 = $ZCPS
if ($?SCALED) echo "$ZCPS1" > NORM.${ID}
#
# Write out simple Fortran code to do arithmetic and format table.
#
cat << EOF > Table.f
      program table
      read (*,*) N, ZCPS1, WALL, TUSED, ZCPS, C90_ZC, C90_MF
      write (*,1) N, WALL, TUSED, NINT(ZCPS), ZCPS*C90_MF/C90_ZC, 
     1            ZCPS/C90_ZC, ZCPS/ZCPS1, ZCPS/(ZCPS1*N)
    1 format(3x,I4,5x,F10.2,1x,F10.2,1x,I12,4x,F8.2,1x,F6.2,3x,F6.2,
     1       7x,F5.2)
      end
EOF
$Fortran -o Table.x Table.f
echo "$N_Threads $ZCPS1 $WALL $TUSED $ZCPS $C90_ZC $C90_MF" > TBL_IN
Table.x < TBL_IN >> $ZBM_Z
endif
#
# Loop over number of threads (if more than one -- runs could fail)
#
#if ($N_THREADS_MIN != 1) then
#  set N_Threads = $N_THREADS_MIN
#else
#  set N_Threads = 2
#endif
#
if (! $?SCALED) @ N_Threads = $N_THREADS_MIN + 1
if ($N_THREADS_MAX > 1) then
#
while ($N_Threads <= $N_THREADS_MAX)
#
# Run ZEUS-3D on N_Threads threads
#
  setenv MP_SET_NUMTHREADS $N_Threads
  echo ""
  echo "N_Threads = $N_Threads"
  if ("$HOST_NAME" == lena) then
    if ($N_Threads < 9) then
      mpa -min $N_Threads -max $N_Threads xzeus34 > ZBM_out
    else if ($N_Threads < 17) then
      @ H2 = $N_Threads - 8
      mpa -min $N_Threads -max $N_Threads xzeus34 > ZBM_out
    else
      @ H3 = $N_Threads - 16
      mpa -min $N_Threads -max $N_Threads xzeus34 > ZBM_out
    endif
  else if ("$HOST_NAME" == ella || "$HOST_NAME" == tuvok) then
    setenv MP_NUMBER_OF_THREADS $N_Threads
    mpa xzeus34 > ZBM_out
  else if ("$HOST_NAME" == billie || "$HOST_NAME" == dax) then
    setenv MP_NUMBER_OF_THREADS $N_Threads
    if ($N_Threads <= 16) then
      mpa -DATA -STACK -private node_private -stacktype node_private xzeus34 > ZBM_out
    else
      mpa -DATA -STACK xzeus34 > ZBM_out
    endif
  else if ("$HOST_NAME" == mario) then
    (hpm xzeus34) >& ZBM_out
  else
    xzeus34 > ZBM_out
  endif
#
# Extract tused
#
  set LINA = (`grep 'tused' ZBM_out | tail -1`)
  if ("$#LINA" > 0) then
    echo "$LINA"
    set TEST = `echo "$LINA" | grep '=-'`
    if ("$TEST" != "") then
#
# Negative trem
#
      @ NLINA = $#LINA - 1
    else
#
# Positive trem
#
      @ NLINA = $#LINA - 2
    endif
    set TUSED = `echo $LINA[$NLINA] | sed -e 's/tused=//'`
#
# Extract the zone-cycles/sec
#
    set LINA = (`grep 'Average   zone-cycles' ZBM_out`)
    echo "$LINA"
    set ZCPS = $LINA[$#LINA]
    if ($?SCALED) set ZCPS1 = `cat NORM.${ID}`
#
# Extract the wall clock time
#
    set LINA = (`grep 'Wall clock time' ZBM_out`)
    echo "$LINA"
    set WALL = `echo $LINA[7] | sed -e 's/,//'`
#
# Write to the table
#
    echo "$N_Threads $ZCPS1 $WALL $TUSED $ZCPS $C90_ZC $C90_MF" > TBL_IN
    Table.x < TBL_IN >> $ZBM_Z
  endif
  @ N_Threads = $N_Threads * 2
end
endif
#
# Finish up
#
echo "" >> $ZBM_Z
echo ""
#
if (! $?SCALED) then
  cat $ZBM_Z
  \rm -f Table.f Table.o Table.x
endif
\rm -f TBL_IN
exit
