#!/bin/csh
#
# HYDRO_FIXED [-c NZX1 NZX2 NZX3 N_THREADS_MAX]
#
#   Compile ZEUS-3D and optionally run the blast hydro test problem 
#   on various grids and numbers of threads.  Prints a nice table.
#
#   Executables and input files for each grid are compiled and renamed.
#
#   Use with ZEUS-3D setup block blast_HY and worker script HYDRO_BM.s.
#
# OPTIONS
#      -c     compile only.  
#
# ARGUMENTS
#
#    NZX*     Number of zones in the full mesh along the "*" axis.
#             Default is 128.
#
#    N_THREADS_MAX    Maximum number of threads to run on.
#
# ENVIRONMENT
#
#    NZX* and NTHREADS_MAX can also be specified through environment
#    variables.  LDR and ID can also be specified this way.
#
# EXAMPLE
#
#    HYDRO_FIXED.s -c 128 64 64
#
#       Compiles a ZEUS-3D executable for a 128x64x64 grid, if one
#       called xz_b_hy_128_64_64 does not exist.  Without
#       the -c option, it would also run ZEUS-3D on 1 to the
#       default maximum number of threads.
#
# RAF 7/16/96
#...................................................................
#
# Check for compile-only flag
#
set RUN = YES
if ($#argv > 0) then
  if ("$1" == "-c") then
    set RUN = NO
    shift
  endif
endif
if ($#argv > 0) then
  setenv NZX1 $1
  shift
else
  if (! $?NZX1) setenv NZX1 128
endif
if ($#argv > 0) then
  setenv NZX2 $1
  shift
else
  if (! $?NZX2) setenv NZX2 128
endif
if ($#argv > 0) then
  setenv NZX3 $1
  shift
else
  if (! $?NZX3) setenv NZX3 128
endif
if ($#argv > 0) then
  setenv N_THREADS_MAX $1
  shift
else
  if (! $?N_THREADS_MAX) setenv N_THREADS_MAX ""
endif
#
# Disable interactive display/edit in zeus34.s
#
setenv ENVIRONMENT BATCH
setenv ID $$
#
# Set variables to turn off NCAR, HDF
#
setenv HDF ""
setenv NCAR_LIB "ncard"
if (! $?HOST_NAME) setenv HOST_NAME `hostname | sed -e 's/\..*//'`
switch ($HOST_NAME)
case mario:
  if (! $?LDR) setenv LDR segldr
  breaksw
case bragi:
case freya:
case loki:
case odin:
case sif:
case thor:
  if (! $?LDR) setenv LDR f77 
  breaksw
case magna:
  if (! $?LDR) setenv LDR "f77 -64 -r10000"
  breaksw
default:
  if (! $?LDR) setenv LDR f77
  breaksw
endsw
#
if (-e chgzeus)    \rm chgzeus
if (-e inzeus)     \rm inzeus
#
set GRID = "${NZX1}_${NZX2}_${NZX3}"
set INP = "in_b_${GRID}"
#
# Do HY only
#
set EXE = "xz_b_hy_${GRID}"
if (-e $EXE) then
  cp $EXE xzeus34
  cp $INP inzeus
else
#
# Compile using all defaults
#
  echo "I need to compile ZEUS-3D to make $EXE"
  echo ""
  if (! -e blast_HY) then
    echo "However, I need the setup block blast_HY on our server in" 
    echo "/afs/ncsa/projects/lca/codes/zeus3d/tests/blast/XYZ"
    exit 1
  endif
  if (-e /usr/afsws/bin/klog && \
    ! -r /afs/ncsa/projects/lca/codes/zeus3d/source/zeus34.s) \
    /usr/afsws/bin/klog -pr zeus3d -pa DDDVets -cell ncsa.uiuc.edu
  zeus34.s blast_HY
  cp xzeus34 $EXE
  cp inzeus $INP
endif
#
# Run N_THREADS_MAX experiments
#
if ($RUN == YES) HYDRO_BM.s $N_THREADS_MAX 0 ${NZX1}x${NZX2}x${NZX3} 
#
exit
